
#include <TimerOne.h>

//include the library code
#include <AccelStepper.h> //stepper motors library

//include the library code
#include <Servo.h> //servo motor library

//Define pins used for the Servo Motor
const int ServoPin = 7;

//Define pins used for the Ultra-Sonic Sensor
const int trigPin = A1;
const int echoPin = A0;


Servo myServo; // create servo object to control a servo

int MaxAngle = 0; // variable to store the servo position
int MaxDistance =0; // variable to store the maximum distance

int Distance =0;

int CurrentDistance =0; //distance measure at each stop

const int MinDistance = 40; //distance covered before scanning

//define some contant numbers that will be used in the code
const int MaxSpeedSlow = 300;
const int MaxSpeedFast = 500;
const int AccelerationSlow = 200;
const int ForwardDistance = 200;



bool obstacle = false;

bool wall = false;

bool stopped = true;

const int Initialize_Distance = 50;

// Define some steppers and the pins they will use
AccelStepper stepper1(1, 8, 9); 
AccelStepper stepper2(1, 5, 6); 
AccelStepper stepper3(1, 3, 2); 
AccelStepper stepper4(1, A2, 4); 


void doSteps()
{
  if(stopped == false)
  {
    stepper1.runSpeed();
    stepper2.runSpeed();
    stepper3.runSpeed();
    stepper4.runSpeed();
  }
    // Toggle LED
    digitalWrite( 13, digitalRead( 13 ) ^ 1 );    
}

void setup() 
{
  Serial.begin (9600);

  pinMode(13,OUTPUT);
  
  //Define inputs and outputs
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  //Define servo pin
  myServo.attach(7);

  //initialize servo position
  myServo.write(90);
  
  delay (2000);

  //set initial stepper settings
  stepper1.setMaxSpeed(MaxSpeedSlow);
  stepper1.setAcceleration(AccelerationSlow);
  
  stepper2.setMaxSpeed(MaxSpeedSlow);
  stepper2.setAcceleration(AccelerationSlow);
  
  stepper3.setMaxSpeed(MaxSpeedSlow);
  stepper3.setAcceleration(AccelerationSlow);
  
  stepper4.setMaxSpeed(MaxSpeedSlow);
  stepper4.setAcceleration(AccelerationSlow);
  
  Timer1.initialize(250); // set a timer of length 250 microseconds 
  Timer1.attachInterrupt( doSteps ); // attach the service routine here
  
  //Initialize Robot
  stepper1.moveTo(-Initialize_Distance);
  stepper2.moveTo(Initialize_Distance);
  stepper3.moveTo(Initialize_Distance);
  stepper4.moveTo(-Initialize_Distance);

  CurrentDistance = measuredDistance();

  delay (2000);  

}

void forward() 
{
  //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(-MaxSpeedSlow);
}

void back() 
{
  //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(MaxSpeedSlow);
}


void Stop() 
{
  stepper1.stop();
  stepper2.stop();
  stepper3.stop();
  stepper4.stop();
}

void rotateLeft() 
{
 //set stepper settings
  stepper1.setSpeed(MaxSpeedSlow);
  
  stepper2.setSpeed(MaxSpeedSlow);
  
  stepper3.setSpeed(MaxSpeedSlow);
  
  stepper4.setSpeed(MaxSpeedSlow);  
}

void rotateRight() 
{
 //set stepper settings
  stepper1.setSpeed(-MaxSpeedSlow);
  
  stepper2.setSpeed(-MaxSpeedSlow);
  
  stepper3.setSpeed(-MaxSpeedSlow);
  
  stepper4.setSpeed(-MaxSpeedSlow); 
}

int measuredDistance()
{
  digitalWrite(trigPin, LOW); // Clears the trigPin
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH); // Sets the trigPin on HIGH state for 10 micro seconds
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);  
 
  // Reads the echoPin, returns the sound wave travel time in microseconds
  long duration = pulseIn(echoPin, HIGH); 

  // Calculating the distance
  long distance = duration /29/2;
  
  return distance;
}

int AngleToGo()
{
  for (int i=0; i<6; i++)
  {
    myServo.write(i*45);
    delay (1000);
    Distance = measuredDistance();
    Serial.println(Distance);
    if (Distance >MaxDistance)
    {
      MaxDistance = Distance;
      MaxAngle = i*45;
    }
    
  }
  return MaxAngle;  
}


void loop() 
{
  CurrentDistance = measuredDistance();
  
  if (CurrentDistance > MinDistance)
  {
    forward();
    stopped = false;    
  }
  else
    {
      Stop();
      stopped = true;
    }
    
}




    
    
  
